
Listing 5: LDAP mit Perl abfragen

#!/usr/bin/perl -w

use strict;
use Net::LDAP;

my $ldapserver = 'ldap.example.net';
my $bindDN       = 'cn=manager,dc=example,dc=net';
my $bindPW	 = 'geheim';	# bitte verstecken!
my $clustername	 = 'postKisten';
my $aliasBase 	 = 'ou=Sendmail, dc=example,dc=net';

my $ldap = Net::LDAP->new($ldapserver, port => 389);

my ($msg, @entries, $e, $filter, $ed);
my ($key, @vals);
my ($rc);

$rc = $ldap->bind("$bindDN", password => "$bindPW");
$rc->code && die "Cannot bind: " . $rc->error;

if (@ARGV == 0) {
    $filter = "(objectClass=sendmailMTAAlias)";
    $ed = 0;
} else {
    $key = $ARGV[0];
    $filter = "(&(objectClass=sendmailMTAAlias)(sendmailMTAKey=$key))";
    $ed = 1;
}

$msg = $ldap->search(
  base   => $aliasBase,
  scope  => 'sub',
  deref  => 'always',
  filter => $filter,
  attrs  => [ 'sendmailMTAKey', 'sendmailMTAAliasValue' ],
	);

$msg->code &&
   die ("search on [$filter] failed with: ",
        $msg->error(),"\n");

if ((@entries = $msg->entries) == 0 && $ed) {
	add_alias();
	exit 0;
}

foreach $e (@entries) {
  my $op;

  $key = $e->get_value('sendmailMTAKey');
  @vals = $e->get_value('sendmailMTAAliasValue');

  printf "%s:\t%s\n", $key, join(", ", @vals);
  if ($ed) { # Es soll editiert werden ?
  	@vals = getvalues();
  	if (@vals != 0) {
  	  if ($vals[0] eq 'delete') {
  	  	$op = 'delete';
  	  	$rc = $ldap->delete($e->dn);
  	  } else {
  	  	$op = 'modify';
  	  	$rc = $ldap->modify($e->dn,
  	         replace =>
		 { 'sendmailMTAAliasValue' => [ @vals ]} );
  	  }
  	  $rc->code && die "failed to $op entry: ", $rc->error ;
  	}
  }
}

# Blank-separierte Werte eingeben
sub getvalues {
	print "Enter values for alias [$key]: ";
	@vals = split(/\s+/, <STDIN>);
}

# Alias im Verzeichnis speichern.
sub add_alias {
    @vals = getvalues();

    $rc = $ldap->add(
	dn => "sendmailMTAKey=$key,$aliasBase",
	attr => [
	  'sendmailMTAKey' => "$key",
	  'sendmailMTACluster' => $clustername,
	  'sendmailMTAAliasValue' => [ @vals ],
	  'sendmailMTAAliasGrouping' => 'aliases',
	  'objectClass' => [ 'sendmailMTA',
	  		     'sendmailMTAAlias',
			     'sendmailMTAAliasObject'
			   ],
		]);
	$rc->code && die "add entry: ", $rc->error ;
}
